/* This file is part of cfpeek
   Copyright (C) 2011 Sergey Poznyakoff

   Cfpeek is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   Cfpeek is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with cfpeek.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <string.h>
#include <errno.h>
#include <sysexits.h>
#include "grecs.h"
#include "wordsplit.h"

int file_index;
int flags = GRECS_NODE_FLAG_PATH|GRECS_NODE_FLAG_VALUE;
int quiet_option;
int all_option;
int reduce_option;
int sort_option;
unsigned max_matches;

#include "cmdline.h"

static void
_print_diag(grecs_locus_t *locus, int err, int errcode, const char *msg)
{
	fflush(stdout);
	if (!locus)
		fprintf(stderr, "%s: ", program_name);
	if (locus)
		fprintf(stderr, "%s:%d: ", locus->file, locus->line);
	if (!err)
		fprintf(stderr, "warning: ");
	fprintf(stderr, "%s", msg);
	if (errcode)
		fprintf(stderr, ": %s", strerror(errno));
	fputc('\n', stderr);
}

static int
node_ident_cmp(struct grecs_node const *a, struct grecs_node const *b)
{
	return strcmp(a->ident, b->ident);
}

int
main(int argc, char **argv)
{
	struct grecs_node *tree;
	int rc = 0;

	grecs_print_diag_fun = _print_diag;
	parse_options(argc, argv);
	
	argc -= file_index;
	argv += file_index;
	if (argc < 1) {
		grecs_error(NULL, 0, "not enough arguments");
		exit(EX_USAGE);
	}
	tree = grecs_parse(argv[0]);
	if (!tree)
		exit(1);
	if (cmdline_tree) {
		grecs_tree_join(tree, cmdline_tree);
		grecs_tree_free(cmdline_tree);
	}
	if (reduce_option)
		grecs_tree_reduce(tree, NULL, 0);
	if (sort_option)
		grecs_tree_sort(tree, node_ident_cmp);
	if (all_option) {
		if (argc > 1) {
			grecs_error(NULL, 0, "too many arguments for --all");
			exit(EX_USAGE);
		}
		grecs_format_node(tree, flags, stdout);
		putc('\n', stdout);
		exit(0);
	}

	if (argc < 2) {
		grecs_error(NULL, 0, "not enough arguments");
		exit(EX_USAGE);
	}
	
	while (--argc) {
		char *arg = *++argv;
		struct grecs_node *node = grecs_find_node(tree, arg);
		if (!node) {
			if (!quiet_option)
				grecs_error(NULL, 0, "%s: not found", arg);
			rc = 1;
		} else if (quiet_option > 1)
			continue;
		else {
			int i = 0;
			do {
				grecs_format_node(node, flags, stdout);
				fputc('\n', stdout);
				if (max_matches && ++i == max_matches)
					break;
			} while (node = grecs_find_node(node->next, arg));
		}
	}
	exit(rc);
}
